/* Virtuino Iot example: ESP8266 Send sensor values to Virtuino app
 * Supported boards: all ESP8266 boards
 * 
 * This example contains the following features
 * 1. How to send sensor values to Virtuino app periodically
 * 2. How to send analog value to Virtuino app periodically
 
 * Created by Ilias Lamprou
 * Updated Mar 10 2022
 */
 
#include <ESP8266WiFi.h>
#include <WebSocketsServer.h> // Download the library WebSockets by Markus Sattler from arduino library manager (ver 2.3.5)

//--- SETTINGS ----------------------------------------
  const char* ssid = "WiFi Name";                      // WIFI network SSID
  const char* password = "WiFi Password";              // WIFI network PASSWORD
  
  WebSocketsServer webSocket = WebSocketsServer(8000); // Default Virtuino Server port = 8000 
  //IPAddress ip(192, 168, 1, 150);                    // where 150 is the desired IP Address. The first three numbers must be the same as the router IP
  //IPAddress gateway(192, 168, 1, 1);                 // set gateway to match your network. Replace with your router IP

//---- Enter all the Tags here. You have to use the same Tags on Virtuino variables
  const char* pinA0_tag= "V9";        // tag for analog input
  const char* sensor1_tag= "V10";     // tag for sensor1 value
  const char* sensor2_tag= "V11";     // tag for sensor2 value

  unsigned long lastSensorRedingTime=0; 
  unsigned long lastAnalogRedingTime=0; 


//===================================================== sendPinStatus
// It is called every time a new client is connected.
// The void informs Virtuino app about the current pin states and variable values.

void sendPinsStatus(){
  sendValue(pinA0_tag, String(analogRead(A0)));    // send the analog value of A0 
  // add more here...
  }

//===================================================== onValueReceived
// It is called every time a new value received
  void onValueReceived(String tag, String value){
    // this example doesn't read values from Virtuino app
    
    }

//===================================================== setup
//=====================================================
  void setup() {
      Serial.begin(115200);
      while (!Serial) delay(1);
      connectToWiFiNetwork();
      webSocket.begin();
      webSocket.onEvent(webSocketEvent);
  }

//===================================================== loop
//=====================================================
  void loop() {
      webSocket.loop();     
  
      //---Example: How to send sensor values to Virtuino every 5 seconds.
      if (millis()-lastSensorRedingTime > 5000) {
        int sensor1 = random(50);                     // replace with your sensor 1 value
        float sensor2 = 5.67;                         // replace with your sensor 2 value
        sendValue(sensor1_tag, String(sensor1));      // send sensor 1 value
        sendValue(sensor2_tag, String(sensor2,2));    // send sensor 2 value
        lastSensorRedingTime=millis();                // store the last sending time of the sensor values
      }

      //---Example: How to send Analog pin values to Virtuino every 8 seconds.
      if (millis()-lastAnalogRedingTime > 8000) {
        sendValue(pinA0_tag, String(analogRead(A0))); // send the analog input value of A0
        lastAnalogRedingTime=millis();                // store the last sending time of analog value
      }

  }


/*=====================================================
  ======================  UTILS =======================
  =====================================================
  You don't need to make changes to the code below
*/
//===================================================== connectToWiFiNetwork
void connectToWiFiNetwork(){
  Serial.println("Connecting to "+String(ssid));
  // If you don't want to config IP manually disable the next two lines and the disables IPAddress variables  at the top of the code
  //IPAddress subnet(255, 255, 255, 0);        // set subnet mask to match your network
  //WiFi.config(ip, gateway, subnet);          // If you don't want to config IP manually disable this line
  WiFi.mode(WIFI_STA);                         // Configure the module as station only.
  WiFi.begin(ssid, password);
   while (WiFi.status() != WL_CONNECTED) {
     delay(500);
     Serial.print(".");
    }
   Serial.println("\nWiFi connected");
   Serial.println(WiFi.localIP());              // Insert this IP on Virtuino IoT server settings
}

 
//===================================================== sendValue
// This function sends a value to a Tag. 
// The tag and the value are converted to a json message before sending 

bool sendValue(const char* tag, String value){
  String json = "{\"";
  json+=tag;
  json+="\":\"";
  json+=value;
  json+="\"}";
  Serial.println("Send: "+json);
  return webSocket.broadcastTXT(json);     // This function sends the message to all connected clients.
}

//===================================================== webSocketEvent
//This is the server handler. It receives all the messages

void webSocketEvent(uint8_t num, WStype_t type, uint8_t * payload, size_t length) {
    switch(type) {
        case WStype_DISCONNECTED:
            Serial.printf("[%u] Disconnected!\n", num);
            break;
        case WStype_CONNECTED:{ 
            IPAddress ip = webSocket.remoteIP(num);
            Serial.printf("[%u] New client connected - IP: %d.%d.%d.%d \n", num, ip[0], ip[1], ip[2], ip[3]);
            sendPinsStatus();         // send the initial pin and variable values to the connected clients
            break;
        }
        case WStype_TEXT:  // a new message received 
            Serial.printf("[%u] Received: %s\n", num, payload);
            //-- The incoming payload is a json message. The following code extracts the value from json without extra library
            String str = (char*)payload;
            int p1 = str.indexOf("{\"");
            if (p1==0) {
              int p2 = str.indexOf("\":");
              if (p2>p1) {
                String tag = str.substring(p1+2,p2);
                p1 = str.indexOf(":\"",p2);
                if (p1>0) {
                   p2 = str.lastIndexOf("\"}");  
                   if (p2>0){
                      String value = str.substring(p1+2,p2);
                      onValueReceived(tag,value);        
                    }
                  }
                }
            }
            break;
    }
}
